@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1999-2020. All Rights Reserved.
+ * Copyright Ericsson AB 1999-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -53,6 +53,7 @@
 #include "erl_thr_progress.h"
 #include "erl_bif_unique.h"
 #include "erl_map.h"
+#include "erl_global_literals.h"
 
 #if 0
 #define DEBUG_PRINTOUTS
@@ -684,7 +685,7 @@ trace_sched_aux(Process *p, ErtsProcLocks locks, Eterm what)
 	curr_func = 0;
     else {
 	if (!p->current)
-	    p->current = find_function_from_pc(p->i);
+	    p->current = erts_find_function_from_pc(p->i);
 	curr_func = p->current != NULL;
     }
 
@@ -948,12 +949,11 @@ seq_trace_output_generic(Eterm token, Eterm msg, Uint type,
  * or   {trace, Pid, return_to, {Mod, Func, Arity}}
  */
 void 
-erts_trace_return_to(Process *p, BeamInstr *pc)
+erts_trace_return_to(Process *p, ErtsCodePtr pc)
 {
+    const ErtsCodeMFA *cmfa = erts_find_function_from_pc(pc);
     Eterm mfa;
 
-    ErtsCodeMFA *cmfa = find_function_from_pc(pc);
-
     if (!cmfa) {
 	mfa = am_undefined;
     } else {
@@ -1122,7 +1122,7 @@ erts_call_trace(Process* p, ErtsCodeInfo *info, Binary *match_spec,
 	 *     use process flags
 	 */
 	tracee_flags = &ERTS_TRACE_FLAGS(p);
-        /* Is is not ideal at all to call this check twice,
+        /* It is not ideal at all to call this check twice,
            it should be optimized so that only one call is made. */
         if (!is_tracer_enabled(p, ERTS_PROC_LOCK_MAIN, &p->common, &tnif,
                                TRACE_FUN_ENABLED, am_trace_status)
@@ -1246,7 +1246,7 @@ erts_call_trace(Process* p, ErtsCodeInfo *info, Binary *match_spec,
     ASSERT(!ERTS_TRACER_IS_NIL(*tracer));
 
     /*
-     * Build the the {M,F,A} tuple in the local heap.
+     * Build the {M,F,A} tuple in the local heap.
      * (A is arguments or arity.)
      */
 
@@ -1387,8 +1387,8 @@ trace_gc(Process *p, Eterm what, Uint size, Eterm msg)
 }
 
 void 
-monitor_long_schedule_proc(Process *p, ErtsCodeMFA *in_fp,
-                           ErtsCodeMFA *out_fp, Uint time)
+monitor_long_schedule_proc(Process *p, const ErtsCodeMFA *in_fp,
+                           const ErtsCodeMFA *out_fp, Uint time)
 {
     ErlHeapFragment *bp;
     ErlOffHeap *off_heap;
@@ -1941,7 +1941,7 @@ profile_runnable_proc(Process *p, Eterm status){
     Eterm *hp, msg;
     Eterm where = am_undefined;
     ErlHeapFragment *bp = NULL;
-    ErtsCodeMFA *cmfa = NULL;
+    const ErtsCodeMFA *cmfa = NULL;
 
     ErtsThrPrgrDelayHandle dhndl;
     Uint hsz = 4 + 6 + patch_ts_size(erts_system_profile_ts_type)-1;
@@ -1955,7 +1955,7 @@ profile_runnable_proc(Process *p, Eterm status){
         if (p->current) {
             cmfa = p->current;
         } else {
-            cmfa = find_function_from_pc(p->i);
+            cmfa = erts_find_function_from_pc(p->i);
         }
     }
 
@@ -2473,7 +2473,7 @@ init_sys_msg_dispatcher(void)
 {
     erts_thr_opts_t thr_opts = ERTS_THR_OPTS_DEFAULT_INITER;
     thr_opts.detached = 1;
-    thr_opts.name = "sys_msg_dispatcher";
+    thr_opts.name = "erts_smsg_disp";
     init_smq_element_alloc();
     sys_message_queue = NULL;
     sys_message_queue_end = NULL;
@@ -2633,6 +2633,9 @@ lookup_tracer_nif(const ErtsTracer tracer)
 {
     ErtsTracerNif tnif_tmpl;
     ErtsTracerNif *tnif;
+    if (tracer == erts_tracer_nil) {
+        return NULL;
+    }
     tnif_tmpl.module = ERTS_TRACER_MODULE(tracer);
     ERTS_LC_ASSERT(erts_thr_progress_lc_is_delaying() || erts_get_scheduler_id() > 0);
     erts_rwmtx_rlock(&tracer_mtx);
@@ -2794,8 +2797,12 @@ send_to_tracer_nif_raw(Process *c_p, Process *tracee,
             map_values[map_elem_count++] = am_monotonic;
 
         map->size = map_elem_count;
-        map->keys = make_tuple(local_heap);
-        local_heap[0] = make_arityval(map_elem_count);
+        if (map_elem_count == 0) {
+            map->keys = ERTS_GLOBAL_LIT_EMPTY_TUPLE;
+        } else {
+            map->keys = make_tuple(local_heap);
+            local_heap[0] = make_arityval(map_elem_count);
+        }
 
 #undef MAP_SIZE
         erts_nif_call_function(c_p, tracee ? tracee : c_p,
@@ -2865,6 +2872,8 @@ is_tracer_enabled(Process* c_p, ErtsProcLocks c_p_locks,
                   enum ErtsTracerOpt topt, Eterm tag) {
     Eterm nif_result;
 
+    ASSERT(t_p);
+
 #if defined(ERTS_ENABLE_LOCK_CHECK)
     if (c_p)
         ERTS_LC_ASSERT(erts_proc_lc_my_proc_locks(c_p) == c_p_locks
@@ -3070,7 +3079,7 @@ erts_tracer_update(ErtsTracer *tracer, const ErtsTracer new_tracer)
     }
 }
 
-static void init_tracer_nif()
+static void init_tracer_nif(void)
 {
     erts_rwmtx_opt_t rwmtx_opt = ERTS_RWMTX_OPT_DEFAULT_INITER;
     rwmtx_opt.type = ERTS_RWMTX_TYPE_EXTREMELY_FREQUENT_READ;
@@ -3083,7 +3092,7 @@ static void init_tracer_nif()
 
 }
 
-int erts_tracer_nif_clear()
+int erts_tracer_nif_clear(void)
 {
 
     erts_rwmtx_rlock(&tracer_mtx);
