@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 1996-2022. All Rights Reserved.
+ * Copyright Ericsson AB 1996-2023. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -89,7 +89,6 @@ extern void erl_sys_args(int*, char**);
 
 extern void erts_sys_init_float(void);
 
-
 #ifdef DEBUG
 static int debug_log = 0;
 #endif
@@ -128,7 +127,7 @@ erts_atomic32_t erts_break_requested;
 
 
 /* set early so the break handler has access to initial mode */
-static struct termios initial_tty_mode;
+struct termios erl_sys_initial_tty_mode;
 static int replace_intr = 0;
 /* assume yes initially, ttsl_init will clear it */
 int using_oldshell = 1;
@@ -162,7 +161,7 @@ void sys_tty_reset(int exit_code)
     SET_BLOCKING(0);
   }
   else if (isatty(0)) {
-    tcsetattr(0,TCSANOW,&initial_tty_mode);
+    tcsetattr(0,TCSANOW,&erl_sys_initial_tty_mode);
   }
 }
 
@@ -240,19 +239,9 @@ thr_create_prepare_child(void *vtcdp)
     erts_lcnt_thread_setup();
 #endif
 
-#ifndef NO_FPE_SIGNALS
-    /*
-     * We do not want fp exeptions in other threads than the
-     * scheduler threads. We enable fpe explicitly in the scheduler
-     * threads after this.
-     */
-    erts_thread_disable_fpe();
-#endif
-
     erts_sched_bind_atthrcreate_child(tcdp->sched_bind_data);
 }
 
-
 void
 erts_sys_pre_init(void)
 {
@@ -261,13 +250,15 @@ erts_sys_pre_init(void)
     erts_printf_add_cr_to_stdout = 1;
     erts_printf_add_cr_to_stderr = 1;
 
-
     eid.thread_create_child_func = thr_create_prepare_child;
     /* Before creation in parent */
     eid.thread_create_prepare_func = thr_create_prepare;
     /* After creation in parent */
     eid.thread_create_parent_func = thr_create_cleanup,
 
+    /* Must be done really early. */
+    sys_init_signal_stack();
+
 #ifdef ERTS_ENABLE_LOCK_COUNT
     erts_lcnt_pre_thr_init();
 #endif
@@ -322,6 +313,10 @@ erts_sys_pre_init(void)
     /* don't lose it, there will be cake */
 }
 
+void erts_sys_scheduler_init(void) {
+    sys_thread_init_signal_stack();
+}
+
 void
 erl_sys_init(void)
 {
@@ -337,7 +332,7 @@ erl_sys_init(void)
     /* we save this so the break handler can set and reset it properly */
     /* also so that we can reset on exit (break handler or not) */
     if (isatty(0)) {
-	tcgetattr(0,&initial_tty_mode);
+	tcgetattr(0,&erl_sys_initial_tty_mode);
     }
 }
 
@@ -681,9 +676,6 @@ void erts_replace_intr(void) {
 void init_break_handler(void)
 {
    sys_signal(SIGINT,  request_break);
-#ifndef ETHR_UNUSABLE_SIGUSRX
-   sys_signal(SIGUSR1, generic_signal_handler);
-#endif /* #ifndef ETHR_UNUSABLE_SIGUSRX */
    sys_signal(SIGQUIT, generic_signal_handler);
 }
 
@@ -698,6 +690,9 @@ void
 erts_sys_unix_later_init(void)
 {
     sys_signal(SIGTERM, generic_signal_handler);
+#ifndef ETHR_UNUSABLE_SIGUSRX
+   sys_signal(SIGUSR1, generic_signal_handler);
+#endif /* #ifndef ETHR_UNUSABLE_SIGUSRX */
 
     /* Ignore SIGCHLD to ensure orphaned processes don't turn into zombies on
      * death when we're pid 1. */
@@ -785,7 +780,7 @@ void erts_do_break_handling(void)
     }
     else if (isatty(0)) {
       tcgetattr(0,&temp_mode);
-      tcsetattr(0,TCSANOW,&initial_tty_mode);
+      tcsetattr(0,TCSANOW,&erl_sys_initial_tty_mode);
       saved = 1;
     }
 
@@ -819,6 +814,10 @@ void sys_get_pid(char *buffer, size_t buffer_size){
     erts_snprintf(buffer, buffer_size, "%lu",(unsigned long) p);
 }
 
+int sys_get_hostname(char *buf, size_t size)
+{
+    return gethostname(buf, size);
+}
 
 void sys_init_io(void) { }
 void erts_sys_alloc_init(void) { }
@@ -928,7 +927,7 @@ void sys_preload_end(Preload* p)
 */
 int sys_get_key(int fd) {
     int c, ret;
-    unsigned char rbuf[64];
+    unsigned char rbuf[64] = {0};
     fd_set fds;
 
     fflush(stdout);		/* Flush query ??? */
@@ -1062,7 +1061,7 @@ init_smp_sig_notify(void)
 {
     erts_thr_opts_t thr_opts = ERTS_THR_OPTS_DEFAULT_INITER;
     thr_opts.detached = 1;
-    thr_opts.name = "sys_sig_dispatcher";
+    thr_opts.name = "erts_ssig_disp";
 
     if (pipe(sig_notify_fds) < 0) {
 	erts_exit(ERTS_ABORT_EXIT,
@@ -1107,13 +1106,12 @@ static void initialize_darwin_main_thread_pipes(void)
 void
 erts_sys_main_thread(void)
 {
-    erts_thread_disable_fpe();
 #ifdef __DARWIN__
     initialize_darwin_main_thread_pipes();
 #else
     /* Become signal receiver thread... */
 #ifdef ERTS_ENABLE_LOCK_CHECK
-    erts_lc_set_thread_name("signal_receiver");
+    erts_lc_set_thread_name("main");
 #endif
 #endif
     smp_sig_notify(0); /* Notify initialized */
