@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 1999-2021. All Rights Reserved.
+%% Copyright Ericsson AB 1999-2023. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -20,8 +20,13 @@
 
 -module(code_SUITE).
 -export([all/0, suite/0, init_per_suite/1, end_per_suite/1, 
-         versions/1,new_binary_types/1, call_purged_fun_code_gone/1,
-	 call_purged_fun_code_reload/1, call_purged_fun_code_there/1,
+         versions/1,new_binary_types/1,
+         bad_beam_file/1,
+         literal_leak/1,
+         call_purged_fun_code_gone/1,
+         call_purged_fun_code_reload/1,
+         call_purged_fun_code_there/1,
+         call_purged_fun_code_altered/1,
          multi_proc_purge/1, t_check_old_code/1,
          external_fun/1,get_chunk/1,module_md5/1,
          constant_pools/1,constant_refc_binaries/1,
@@ -29,24 +34,28 @@
          false_dependency/1,coverage/1,fun_confusion/1,
          t_copy_literals/1, t_copy_literals_frags/1,
          erl_544/1, max_heap_size/1,
-	 check_process_code_signal_order/1,
-	 check_process_code_dirty_exec_proc/1]).
+         check_process_code_signal_order/1,
+         check_process_code_dirty_exec_proc/1,
+         call_fun_before_load/1]).
 
 -define(line_trace, 1).
 -include_lib("common_test/include/ct.hrl").
 
 suite() -> [{ct_hooks,[ts_install_cth]}].
 
-all() -> 
-    [versions, new_binary_types, call_purged_fun_code_gone,
+all() ->
+    [versions, new_binary_types,
+     bad_beam_file, literal_leak,
+     call_purged_fun_code_gone,
      call_purged_fun_code_reload, call_purged_fun_code_there,
+     call_purged_fun_code_altered,
      multi_proc_purge, t_check_old_code, external_fun, get_chunk,
      module_md5,
      constant_pools, constant_refc_binaries, fake_literals,
      false_dependency,
      coverage, fun_confusion, t_copy_literals, t_copy_literals_frags,
      erl_544, max_heap_size, check_process_code_signal_order,
-     check_process_code_dirty_exec_proc].
+     check_process_code_dirty_exec_proc, call_fun_before_load].
 
 init_per_suite(Config) ->
     erts_debug:set_internal_state(available_internal_state, true),
@@ -135,6 +144,81 @@ new_binary_types(Config) when is_list(Config) ->
                                                     bit_sized_binary(Bin))),
     ok.
 
+%% Ensure that the loader doesn't crash or leak memory when attempting
+%% to load bad BEAM files. We depend on valgrind to notice leaks.
+bad_beam_file(_Config) ->
+    Mod = ?FUNCTION_NAME,
+
+    BadBeam1 = bad_beam_file_1(Mod),
+    {error,badfile} = code:load_binary(Mod, atom_to_list(Mod), BadBeam1),
+
+    BadBeam2 = bad_beam_file_2(Mod),
+    {error,badfile} = code:load_binary(Mod, atom_to_list(Mod), BadBeam2),
+
+    ok.
+
+%% Build a BEAM file with an invalid instruction in the code chunk.
+bad_beam_file_1(Mod) ->
+    Exp = [{term,0}],
+    Attr = [],
+    Fs = [{function,term,0,3,
+           [{label,1},
+            {line,[]},
+            {func_info,{atom,Mod},{atom,term},0},
+            {label,2},
+            {move,nil,nil},                     %Illegal destination.
+            return]}],
+    Asm = {Mod,Exp,Attr,Fs,3},
+
+    %% Bypass beam_validator.
+    {ok,BadBeam} = beam_asm:module(Asm, [], [], []),
+    BadBeam.
+
+%% Build a BEAM file with an invalid attributes chunk.
+bad_beam_file_2(Mod) ->
+    Asm = {Mod,[],[],[],1},
+    {ok,BadBeam0} = beam_asm:module(Asm, [], [], []),
+    {ok, Mod, Chunks0} = beam_lib:all_chunks(BadBeam0),
+    Chunks1 = lists:keydelete("Attr", 1, Chunks0),
+    Chunks = [{"Attr",<<"bad_attribute_chunk">>} | Chunks1],
+    {ok,BadBeam} = beam_lib:build_module(Chunks),
+    BadBeam.
+
+%% Ensure that literal areas don't leak when erlang:prepare_loading/2
+%% is not followed by erlang:finish_loading/1.
+literal_leak(_Config) ->
+    Mod = ?FUNCTION_NAME,
+    HugeLiteral = binary_to_list(<<0:(1024*1024)/unit:8>>),
+    Exp = [{term,0}],
+    Attr = [],
+    Fs = [{function,term,0,3,
+           [{label,1},
+            {line,[]},
+            {func_info,{atom,Mod},{atom,term},0},
+            {label,2},
+            {move,{literal,HugeLiteral},{x,0}},
+            return]}],
+    Asm = {Mod,Exp,Attr,Fs,3},
+    {ok,Beam} = beam_asm:module(Asm, [], [], []),
+
+    %% valgrind cannot help us find leak of literals because literal
+    %% areas are allocated using mmap().
+    %%
+    %% Instead we will prepare for loading a BEAM file with a 16Mb
+    %% literal N times so that the reserved literal memory range will
+    %% overflow.
+    %%
+    %% Setting N to 64 is sufficient for overflowing a 1 GB literal
+    %% area (which is the size at the time of writing). Just to be
+    %% sure, we will go a little bit higher...
+
+    N = 128,
+    _ = [begin
+             _ = erlang:prepare_loading(Mod, Beam),
+             _ = erlang:garbage_collect()
+         end || _ <- lists:seq(1, N)],
+    ok.
+
 call_purged_fun_code_gone(Config) when is_list(Config) ->
     run_sys_proc_test(fun call_purged_fun_code_gone_test/1, Config).
 
@@ -168,24 +252,25 @@ call_purged_fun_code_there_test(Config) when is_list(Config) ->
     call_purged_fun_test(Priv, Data, code_there),
     ok.
 
+%% GH-7288: calling a fun defined by a module that had been purged after
+%% loading a different version of the same module (and therefore did not
+%% inherit the old fun entries) could cause the emulator to crash.
+call_purged_fun_code_altered(Config) when is_list(Config) ->
+    run_sys_proc_test(fun call_purged_fun_code_altered_test/1, Config).
+
+call_purged_fun_code_altered_test(Config) when is_list(Config) ->
+    Priv = proplists:get_value(priv_dir, Config),
+    Data = proplists:get_value(data_dir, Config),
+    call_purged_fun_test(Priv, Data, code_altered),
+    ok.
+
 call_purged_fun_test(Priv, Data, Type) ->
-    OptsList = case erlang:system_info(hipe_architecture) of
-                   undefined -> [[]];
-                   _ -> [[], [native,{d,hipe}]]
-               end,
-    [call_purged_fun_test_do(Priv, Data, Type, CO, FO)
-     || CO <- OptsList, FO <- OptsList].
-
-
-call_purged_fun_test_do(Priv, Data, Type, CallerOpts, FunOpts) ->
-    io:format("Compile caller as ~p and funs as ~p\n", [CallerOpts, FunOpts]),
     SrcFile = filename:join(Data, "call_purged_fun_tester.erl"),
     ObjFile = filename:join(Priv, "call_purged_fun_tester.beam"),
-    {ok,Mod,Code} = compile:file(SrcFile, [binary, report | CallerOpts]),
+    {ok,Mod,Code} = compile:file(SrcFile, [binary, report]),
     {module,Mod} = code:load_binary(Mod, ObjFile, Code),
 
-    call_purged_fun_tester:do(Priv, Data, Type, FunOpts).
-
+    call_purged_fun_tester:do(Priv, Data, Type, []).
 
 multi_proc_purge(Config) when is_list(Config) ->
     run_sys_proc_test(fun multi_proc_purge_test/1, Config).
@@ -504,7 +589,7 @@ constant_refc_binaries_test(Config) when is_list(Config) ->
     Bef = memory_binary(),
     io:format("Binary data (bytes) before test: ~p\n", [Bef]),
 
-    %% Compile the the literals module.
+    %% Compile the literals module.
     Data = proplists:get_value(data_dir, Config),
     File = filename:join(Data, "literals"),
     {ok,literals,Code} = compile:file(File, [report,binary]),
@@ -660,7 +745,7 @@ make_literal_module(Mod, Term) ->
             {label,2},
             {move,{literal,Term},{x,0}},
             return]}],
-    Asm = {Mod,Exp,Attr,Fs,2},
+    Asm = {Mod,Exp,Attr,Fs,3},
     {ok,Mod,Beam} = compile:forms(Asm, [from_asm,binary,report]),
     code:load_binary(Mod, atom_to_list(Mod), Beam).
 
@@ -675,11 +760,11 @@ verify_lit_terms([], _) ->
     ok.
 
 get_external_terms() ->
-    {ok,Node} =	test_server:start_node(?FUNCTION_NAME, slave, []),
+    {ok, Peer, Node}= ?CT_PEER(),
     Ref = rpc:call(Node, erlang, make_ref, []),
     Ports = rpc:call(Node, erlang, ports, []),
     Pid = rpc:call(Node, erlang, self, []),
-    _ = test_server:stop_node(Node),
+    peer:stop(Peer),
     {Ref,hd(Ports),Pid}.
 
 %% OTP-7559: c_p->cp could contain garbage and create a false dependency
@@ -696,24 +781,6 @@ false_dependency_test(Config) when is_list(Config) ->
     do_false_dependency(fun cpbugx:before2/0, Code),
     do_false_dependency(fun cpbugx:before3/0, Code),
 
-    %%     %% Spawn process. Make sure it has called cpbugx:before/0 and returned.
-    %%     Parent = self(),
-    %%     Pid = spawn_link(fun() -> false_dependency_loop(Parent) end),
-    %%     receive initialized -> ok end,
-
-    %%     %% Reload the module. Make sure the process is still alive.
-    %%     {module,cpbugx} = erlang:load_module(cpbugx, Bin),
-    %%     io:put_chars(binary_to_list(element(2, process_info(Pid, backtrace)))),
-    %%     true = is_process_alive(Pid),
-
-    %%     %% There should not be any dependency to cpbugx.
-    %%     false = erlang:check_process_code(Pid, cpbugx),
-
-
-
-
-    %%     %% Kill the process.
-    %%     unlink(Pid), exit(Pid, kill),
     ok.
 
 do_false_dependency(Init, Code) ->
@@ -737,9 +804,7 @@ do_false_dependency(Init, Code) ->
     unlink(Pid), exit(Pid, kill),
     true = erlang:purge_module(cpbugx),
     true = erlang:delete_module(cpbugx),
-    code:is_module_native(cpbugx),  % test is_module_native on deleted code
     true = erlang:purge_module(cpbugx),
-    code:is_module_native(cpbugx),  % test is_module_native on purged code
     ok.
 
 false_dependency_loop(Parent, Init, SendInitAck) ->
@@ -757,9 +822,7 @@ false_dependency_loop(Parent, Init, SendInitAck) ->
     end.
 
 coverage(Config) when is_list(Config) ->
-    code:is_module_native(?MODULE),
     {'EXIT',{badarg,_}} = (catch erlang:purge_module({a,b,c})),
-    {'EXIT',{badarg,_}} = (catch code:is_module_native({a,b,c})),
     {'EXIT',{badarg,_}} = (catch erlang:check_process_code(not_a_pid, ?MODULE)),
     {'EXIT',{badarg,_}} = (catch erlang:check_process_code(self(), [not_a_module])),
     {'EXIT',{badarg,_}} = (catch erlang:delete_module([a,b,c])),
@@ -795,7 +858,7 @@ t_copy_literals(Config) when is_list(Config) ->
     run_sys_proc_test(fun t_copy_literals_test/1, Config).
 
 t_copy_literals_test(Config) when is_list(Config) ->
-    %% Compile the the literals module.
+    %% Compile the literals module.
     Data = proplists:get_value(data_dir, Config),
     File = filename:join(Data, "literals"),
     {ok,literals,Code} = compile:file(File, [report,binary]),
@@ -1047,9 +1110,9 @@ erl_544(Config) when is_list(Config) ->
                 File = proplists:get_value(file, Info2),
                 StackFun = fun(_, _, _) -> false end,
                 FormatFun = fun (Term, _) -> io_lib:format("~tp", [Term]) end,
-                Formated =
+                Formatted =
                     erl_error:format_stacktrace(1, Stack, StackFun, FormatFun),
-                true = is_list(Formated),
+                true = is_list(Formatted),
                 ok
             after
                 ok = file:set_cwd(CWD)
@@ -1135,6 +1198,29 @@ check_process_code_dirty_exec_proc(Config) when is_list(Config) ->
     false = is_process_alive(Pid),
     ok.
 
+%% OTP-18016: When loading a module over itself, a race in fun loading made it
+%% possible to call a local fun before the module was fully reloaded.
+call_fun_before_load(Config) ->
+    Priv = proplists:get_value(priv_dir, Config),
+    Data = proplists:get_value(data_dir, Config),
+    Path = code:get_path(),
+    true = code:add_path(Priv),
+    try
+        SrcFile = filename:join(Data, "call_fun_before_load.erl"),
+        ObjFile = filename:join(Priv, "call_fun_before_load.beam"),
+        {ok,Mod,Code} = compile:file(SrcFile, [binary, report]),
+        {module,Mod} = code:load_binary(Mod, ObjFile, Code),
+
+        ok = call_fun_before_load:run(ObjFile, Code),
+
+        code:purge(call_fun_before_load)
+    after
+        code:set_path(Path),
+        code:delete(call_fun_before_load),
+        code:purge(call_fun_before_load)
+    end,
+    ok.
+
 %% Utilities.
 
 make_sub_binary(Bin) when is_binary(Bin) ->
